/*
 * filename: src/arg_parse.c
 * author: Pablo Yanez Trujillo <yanezp@imtek.de>
 */

#include <getopt.h>
#include <string.h>
#include <stdio.h>

static void help(const char* name, int error)
{
	const char* help = 
"usage: %s [--display device] | [--help]\n\
--display	if you want to start the gui in another\n\
		(non-default) screen of the X11 server.\n\
--help		shows this message\n\
This is free software under the GPL-2 licence\n\
Author: Pablo Yanez Trujillo <yanezp@imtek.de>\n";

	FILE* f = error ? stderr : stdout;

	fprintf(f, help, name);
}

int arg_parse(int argc, char *argv[])
{
	int c;

	while( (c = getopt(argc, argv, " -:")) != -1 )
	{
		switch(c)
		{
			case '-':
				if(!strcmp(optarg, "help"))
				{
					help(argv[0], 0);
					return 2;
				} else {
					fprintf(stderr, "Unknown option `--%s'.\n", optarg);
					help(argv[0], 1);
					return 0;
				}
				break;
			case '?':
			default:
				if (isprint (optopt))
                                        fprintf (stderr, "Unknown option `-%c'.\n", optopt);
                                else    
                                        fprintf (stderr, "Unknown option character `\\x%x'.\n", optopt);
                                
                                help(argv[0], 1);
                                return 0;
		}
	}
	
	return 1;
}
