/*
 * filename: src/callbacks.c
 * author: Pablo Yanez Trujillo <yanezp@imtek.de>
 */

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <gutils.h>
#include "callbacks.h"
#include "db.h"

GladeXML *__xml_dialog;
const char *__image_path;
int __image_type;
int __image_speed;

enum
{
	CD_TYPE = 0,
	DVD_TYPE
};

int do_burn(const char* path, int type, int speed);

void on_mainWindow_destroy(GtkWidget *widget, gpointer data)
{
	gtk_main_quit();
}
void category_selected(GtkWidget *widget, gpointer data)
{
	GtkTreeView* view;
	gchar* catname;
	GString* cat_iso;
	GtkLabel* label;

	GtkComboBox *speedcombo = GTK_COMBO_BOX(glade_xml_get_widget(__xml, "optSpeed"));

	if(gtk_combo_box_get_active(GTK_COMBO_BOX(widget)) == -1) return;

	catname = gtk_combo_box_get_active_text(GTK_COMBO_BOX(widget));

	cat_iso = g_string_utf8_to_iso(catname);

	cat_iso = g_string_trim(cat_iso);

	/* create a list with images under this category 
	 * refresh view on GtkTreeView object
	 */

	view = GTK_TREE_VIEW(glade_xml_get_widget(__xml, "imageView"));
	gtk_clean_treeview(view, __images);

	g_list_string_free(__images);
	__images = get_images(cat_iso->str);

	if(__last_type && *__last_type == 'C')
                gtk_clean_combobox(speedcombo, __cd_speed);
        else if(__last_type && *__last_type == 'D')
                gtk_clean_combobox(speedcombo, __dvd_speed);

	label = GTK_LABEL(glade_xml_get_widget(__xml, "lblDescription"));
	gtk_label_clear(label);


	gtk_fill_treeview(view, __images);

	g_string_free(cat_iso, TRUE);
}
void refresh_view(GtkWidget *widget, gpointer data)
{
	GtkComboBox* combo;
	gint pos;

	combo = GTK_COMBO_BOX(glade_xml_get_widget(__xml, "optCategories"));

	pos = gtk_clean_combobox(combo, __categories);

	pos = pos == -1 ? 0 : pos;

	g_list_string_free(__categories);

	__categories = get_categories();
	gtk_fill_combobox(combo, __categories, pos);
}


void burn(GtkWidget *widget, gpointer data)
{
	GtkWindow* window;
	GtkWidget* dialog;
	GtkComboBox* speedcombo;
	GtkTreeView *treeview;
	gint pos, exe;
	GString* type, *path, *sp;
	GList *argv = NULL, *speedlist;
	gint speed;
	gchar* tmp;

	treeview = GTK_TREE_VIEW(glade_xml_get_widget(__xml, "imageView"));

	pos = gtk_tree_view_get_single_pos(treeview);

	if(pos == -1) return;

	type = g_list_nth_data(__images, 9 * pos + 4);
	path = g_list_nth_data(__images, 9 * pos + 7);


	speedcombo = GTK_COMBO_BOX(glade_xml_get_widget(__xml, "optSpeed"));

	speed = gtk_combo_box_get_active(speedcombo);

	if(speed)
	{
		/* speed is not auto */
		speedlist = strcmp(type->str, "CD") ? __dvd_speed : __cd_speed;
		
		sp = g_list_nth_data(speedlist, speed);
		tmp = sp->str;

		tmp[strlen(tmp) - 1] = 0;

		speed = atoi(tmp);

		tmp[strlen(tmp)] = 'X';
	}

	window = GTK_WINDOW(glade_xml_get_widget(__xml, "mainWindow"));

	dialog = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, "Lege eine leere %s ein", type->str);

	exe = gtk_exec("eject", NULL);

	if(exe)
	{
		
		gtk_widget_destroy (dialog);
		dialog = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
				"Kann die CD-Schublade nicht aufmachen.\n"
				"Informiere bitte einen PoolMgr oder schick\n"
				"eine Mail an poolmgr@informatik.uni-freiburg.de");

		gtk_dialog_run(GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		return;
	}

	gtk_dialog_run(GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);

	argv = g_list_append(argv, (gchar*) "-t");

	exe = gtk_exec("eject", argv);

	if(exe)
        {
                gtk_widget_destroy (dialog);

		dialog = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
				"Kann die CD-Schublade nicht zumachen.\n"
				"Informiere bitte einen PoolMgr oder schick\n"
				"eine Mail an poolmgr@informatik.uni-freiburg.de");

		gtk_dialog_run(GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
                return;
        }

	g_list_free(argv);

	do_burn(path->str, !strcmp(type->str, "CD") ? CD_TYPE : DVD_TYPE, speed);
}

void image_selected(GtkTreeView *treeview, GtkTreePath *arg1, GtkTreeViewColumn *arg2, gpointer user_data)
{
	GtkComboBox* combo;
	GString* data;
	GtkLabel* label;
	gchar* str;
	gint pos;
	
	pos = gtk_tree_view_get_single_pos(treeview);

	combo = GTK_COMBO_BOX(glade_xml_get_widget(__xml, "optSpeed"));

	if(pos == -1)
	{
		if(!__last_type)
			return;

		gtk_clean_combobox(combo, *__last_type == 'C' ? __cd_speed : __dvd_speed);
		__last_type = NULL;
		
		label = GTK_LABEL(glade_xml_get_widget(__xml, "lblDescription"));
		gtk_label_clear(label);
		return;
	}
	
	if(__last_type && *__last_type == 'C')
		gtk_clean_combobox(combo, __cd_speed);
	else if(__last_type && *__last_type == 'D')
		gtk_clean_combobox(combo, __dvd_speed);

	data = g_list_nth_data(__images, 8*pos + 4);

	str = data->str;

	if(*str == 'C')
	{
		__last_type = "CD";
		gtk_fill_combobox(combo, __cd_speed, 0);
	} else {
		__last_type = "DVD";
		gtk_fill_combobox(combo, __dvd_speed, 0);
	}

	label = GTK_LABEL(glade_xml_get_widget(__xml, "lblDescription"));
	gtk_label_set_aof(label, __images, pos);
}

void burn_done(GtkWidget *widget, gpointer data)
{
	GList* ar = NULL;
	GtkWidget *dia = glade_xml_get_widget(__xml_dialog, "burn_dia");

	gtk_widget_destroy(dia);
	
	ar = g_list_append(ar, (gchar*) "-t");

	gtk_exec("eject", ar);

	g_list_free(ar);
}

void burn_prepare(GtkWidget *widget, gpointer data)
{
	int exe = 0;
	GList* ar = NULL;
	GtkWidget* button;
	GtkLabel* message;
	gchar speed[15], growiso[1024];
	const gchar* command = NULL;
	GString* lbl_utf8;
	FILE* log;
	time_t timestamp;
	char* timestr;

	button = glade_xml_get_widget(__xml_dialog, "cmdBurnOk");

	/* we will burn here 
	 * cd burn: /usr/bin/cdrecord -v gracetime=2 dev=/dev/hdc speed=? -dao driveropts=burnfree -data image.iso
	 * dvd burn: /usr/bin/growisofs -Z /dev/hdc=image.iso -use-the-force-luke=notray -use-the-force-luke=tty -dvd-compat -speed=?
	 * speed iff __image_speed != 0 */

	switch(__image_type)
	{
		case CD_TYPE:
			command = "/usr/bin/cdrecord";
			ar = g_list_append(ar, (gchar*) "-v");
			ar = g_list_append(ar, (gchar*) "gracetime=2");
			ar = g_list_append(ar, (gchar*) "dev=/dev/hdc");
			ar = g_list_append(ar, (gchar*) "-dao");
			ar = g_list_append(ar, (gchar*) "driveropts=burnfree");
			ar = g_list_append(ar, (gchar*) "-data");
			ar = g_list_append(ar, (gchar*) __image_path);
			if(__image_speed)
			{
				sprintf(speed, "speed=%d", __image_speed);
				ar = g_list_append(ar, speed);
			}
			break;
		case DVD_TYPE:
			command = "/usr/bin/growisofs";
			sprintf(growiso, "/dev/hdc=%s", __image_path);
			ar = g_list_append(ar, (gchar*) "-Z");
			ar = g_list_append(ar, growiso);
			ar = g_list_append(ar, (gchar*) "-use-the-force-luke=notray");
			ar = g_list_append(ar, (gchar*) "-use-the-force-luke=tty");
			ar = g_list_append(ar, (gchar*) "-dvd-compat");
			if(__image_speed)
			{
				sprintf(speed, "-speed=%d", __image_speed);
				ar = g_list_append(ar, speed);
			}			
			break;
	}

	message = GTK_LABEL(glade_xml_get_widget(__xml_dialog, "lblMessage"));

	if(!ar)
	{
		gtk_label_set_text(message, "Unerwarteter Fehler.\nBitte an poolmgr@informatik.uni-freiburg.de melden!");
		gtk_widget_show(GTK_WIDGET(message));
		
	} else {

		exe = gtk_exec(command, ar);

		if(exe)
		{
			 gtk_label_set_text(message, "Fehler beim Brennvorgang!\n");
			 gtk_widget_show(GTK_WIDGET(message));
			 g_list_free(ar);
			 gtk_widget_set(button, "sensitive", TRUE, NULL);
			 return;
		}

		timestamp = time(NULL);

		if(timestamp != -1)
		{
			timestr = ctime(&timestamp);
			if(timestr)
			{
				timestr[strlen(timestr) - 1] = 0;
				
				log = fopen("/var/log/aof-burn/aof.log", "a");

				if(log)
				{
					fprintf(log, "[%s] %s\n", timestr, __image_path);
					fclose(log);
				}
			}
		}

		
		sprintf(growiso, "Brennvorgang erfolgreich beendet.\nViel Spa mit der %s", __image_type == CD_TYPE ? "CD" : "DVD");
		lbl_utf8 = g_string_iso_to_utf8(growiso);
		
		gtk_label_set_text(message, lbl_utf8->str);

		gtk_widget_show(GTK_WIDGET(message));

		gtk_exec("eject", NULL);
		g_string_free(lbl_utf8, TRUE);
		g_list_free(ar);
	}
		
	gtk_widget_set(button, "sensitive", TRUE, NULL);
}

int do_burn(const char* path, int type, int speed)
{
	const char *glade_file;
	GtkLabel* message;
	GtkWidget* button;
        char mes[100];

#ifdef DEVELOP_VERSION
        glade_file = DEVELOP_VERSION "/aof-gui.glade";
#else
        glade_file = GLADE_FILE "/aof-gui.glade";
#endif

	__image_path = path;
	__image_type = type;
	__image_speed = speed;

	__xml_dialog = glade_xml_new(glade_file, "burn_dia", NULL);
	glade_xml_signal_autoconnect(__xml_dialog);

	message = GTK_LABEL(glade_xml_get_widget(__xml_dialog, "lblMessage"));

        sprintf(mes, "Bitte, warte bis die %s gebrannt ist...", type == CD_TYPE ? "CD" : "DVD");
	gtk_label_set_text(message, mes);
	gtk_widget_show(GTK_WIDGET(message));
	

	gtk_widget_show(GTK_WIDGET(message));

	button = glade_xml_get_widget(__xml_dialog, "cmdBurnOk");

	gtk_widget_set(button, "sensitive", FALSE, NULL);
	
	return 1;
}
