/*
 * filename: src/db.c
 * author: Pablo Yanez Trujillo <yanezp@imtek.de>
 *
 * BIG Database module
 */

#include <stdio.h>

#include <gutils.h>
#include "db.h"


/* I don't like it, but since this is C and I cannot put variables together
 * into a class I have to use global variables, which are implemented on
 * src/db.c
 * */

sqlite3* __db;

sqlite3* db_connect(const char* path)
{
	FILE* f;
	sqlite3* db;
	int rc;
	
	if(!path || !*path) return NULL;

	f = fopen(path, "r");

	if(!f)
	{
		fprintf(stderr, "Cannot open/read the database file '%s'. Did you install aof-db?\n", path);
		return NULL;
	}

	fclose(f);

	rc = sqlite3_open(path, &db);

	if(rc)
	{
		fprintf(stderr, "Can't open database: %s\n", sqlite3_errmsg(db));
		sqlite3_close(db);
		return NULL;
	}

	return db;
}





/****************** get_categories *************************/
static int __get_categories_callback(void *user, int argc, char **argv, char **azColName)
{
        GList *result;
	GString* name_utf8;

	result = user;

	result = g_list_append(result, g_string_new(argv[0]));

	name_utf8 = g_string_iso_to_utf8(argv[1]);

	if(!name_utf8)
		result = g_list_append(result, g_string_new(argv[1]));
	else
		result = g_list_append(result, g_string_new(name_utf8->str));

	g_string_free(name_utf8, TRUE);

	return 0;
}

static void __get_categories(const char* id, int root, GList** list)
{
	GList* result = NULL; 
	char sql_command[1024];
	char* sqlError = NULL;
	int i, rc;
	GString* data;
	GString* indent = g_string_new("");

	if(!indent) return;

	for(i=0; i<root; ++i)
		indent = g_string_append(indent, " ");

	sprintf(sql_command, "SELECT id,name FROM category WHERE subid = %s ORDER BY name", id);

	result = g_list_append(result, g_string_new(""));

	rc = sqlite3_exec(__db, sql_command, __get_categories_callback, result, &sqlError);

	if(rc != SQLITE_OK)
	{
		g_list_string_free(result);
		g_string_free(indent, TRUE);
		return;
	}

	rc = g_list_length(result);

	for(i = 1; i < rc; i = i + 2)
	{
		/* get name */
		data = g_list_nth_data(result, i+1);
		data = g_string_prepend(data, indent->str);
		*list = g_list_append(*list, g_string_new(data->str));

		/* get id */
		data = g_list_nth_data(result, i);
		__get_categories(data->str, root + 3, list);
	}

	g_list_string_free(result);
	g_string_free(indent, TRUE);
}

GList* get_categories()
{
	GList *result = NULL;
	__get_categories("0", 0, &result);

	return result;
}







/****************** get_images *************************/

static int __get_images_callback(void *user, int argc, char **argv, char **azColName)
{
	GList *result;
	GString* name_utf8 = NULL;

	/* id, name, desc, longdesc, type, year, author, path, language
	 * 0   1      2     3        4     5      6      7     8
	 *     +      +     +                     +      +     +
	 */

	result = user;


	result = g_list_append(result, g_string_new(argv[0])); /* id */
	name_utf8 = g_string_iso_to_utf8(argv[1]); /* name */
	g_list_append(result, g_string_new( name_utf8 ? name_utf8->str : argv[1] ));
	g_string_free(name_utf8, TRUE);

	name_utf8 = g_string_iso_to_utf8(argv[2]); /* desc */
	g_list_append(result, g_string_new( name_utf8 ? name_utf8->str : argv[2] ));
	g_string_free(name_utf8, TRUE);

	name_utf8 = g_string_iso_to_utf8(argv[3]); /* longdesc */
	g_list_append(result, g_string_new( name_utf8 ? name_utf8->str : argv[3] ));
	g_string_free(name_utf8, TRUE);
	
	result = g_list_append(result, g_string_new(argv[4])); /* type */
	result = g_list_append(result, g_string_new(argv[5])); /* year */

	name_utf8 = g_string_iso_to_utf8(argv[6]); /* author */
	g_list_append(result, g_string_new( name_utf8 ? name_utf8->str : argv[6] ));
	g_string_free(name_utf8, TRUE);

	name_utf8 = g_string_iso_to_utf8(argv[7]); /* path */
	g_list_append(result, g_string_new( name_utf8 ? name_utf8->str : argv[7] ));
	g_string_free(name_utf8, TRUE);

	name_utf8 = g_string_iso_to_utf8(argv[8]); /* language */
	g_list_append(result, g_string_new( name_utf8 ? name_utf8->str : argv[8] ));
	g_string_free(name_utf8, TRUE);

	return 0;
}

static void __get_images(const gchar* category, GList** list)
{
	GList* result = NULL;
	char sql_command[1024];
	char* sqlError = NULL;
	int i, rc;
	GString* data;

	sprintf(sql_command, "SELECT I.id,I.name,I.desc,I.longdesc,I.type,I.year,I.author,I.path,I.language FROM image I JOIN category C on I.cat = C.id WHERE C.name = '%s' ORDER BY I.name;", category);

	result = g_list_append(result, g_string_new(""));

	rc = sqlite3_exec(__db, sql_command, __get_images_callback, result, &sqlError);

	if(rc != SQLITE_OK)
	{
		g_list_string_free(result);
		return;
	}

	rc = g_list_length(result);

	for(i = 1; i < rc; ++i)
	{
		data = g_list_nth_data(result, i);
		*list = g_list_append(*list, g_string_new(data->str));
	}
	g_list_string_free(result);
}


GList* get_images(const gchar* category)
{
	GList* result = NULL;
	if(!category || !*category) return NULL;

	__get_images(category, &result);

	return result;
}
