/*
 * filename: src/gtk_common.c
 * author: Pablo Yanez Trujillo <yanezp@imtek.de>
 */

#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdlib.h>

#include "gtk_common.h"
#include "db.h"

/* if there is a better solution for this, please
 * please tell me....
 */
GladeXML *__xml;
GList *__categories;
GList *__images;
GList *__cd_speed;
GList *__dvd_speed;
const gchar *__last_type;


gint gtk_clean_combobox(GtkComboBox* combo, GList* list)
{
	gint old_pos;
	guint len, i;

	old_pos = gtk_combo_box_get_active(combo);

	len = g_list_length(list);

	gtk_combo_box_set_active(combo, -1);

	for(i = 0; i < len; ++i)
	{
		gtk_combo_box_remove_text(combo, 0);
	}
	
	return old_pos;
}

static void __combo_append(gpointer data, gpointer user_data)
{
	GString* str = data;
	GtkWidget *combo = user_data;

	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), str->str);
}

void gtk_fill_combobox(GtkComboBox* combo, GList* list, gint position)
{
	gint pos;
	g_list_foreach(list, __combo_append, combo);

	if(position < g_list_length(list))
		pos = position;
	else pos = 0;

	gtk_combo_box_set_active(combo, pos);
}


void gtk_clean_treeview(GtkTreeView* view, GList* list)
{
	int i,len;
	GtkTreeModel* model;
	GtkTreePath* path;
	GtkTreeIter iter;

	model = gtk_tree_view_get_model(view);

	path = gtk_tree_path_new_from_string("0");

	len = g_list_length(list) / 8;

	for(i = 0; i < len; ++i)
	{
		if(!gtk_tree_model_get_iter_from_string(model, &iter, "0"))
			return;

		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
	}

	gtk_tree_path_free(path);
}


void gtk_fill_treeview(GtkTreeView* view, GList* list)
{
	int i,len;
	GtkListStore* store;
	GtkTreeIter iter;
	GString *name, *type, *desc, *lang;
	GtkTreeModel* model;
	GtkCellRenderer *renderer;
	GtkTreePath* path = NULL;
	static int firstrun = 1;

	/* id, name, desc, longdesc, type, year, author, path, language
         * 0   1      2     3        4     5      6      7     8
         *     +      +     +                     +      +     +
         */

	store = gtk_list_store_new(NUM_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

	len = g_list_length(list) / 9;

	for(i = 0; i < len; ++i)
	{
		name = g_list_nth_data(list, i*9 + 1);
		type = g_list_nth_data(list, i*9 + 4);
		desc = g_list_nth_data(list, i*9 + 2);
		lang = g_list_nth_data(list, i*9 + 8);

		gtk_list_store_append (store, &iter);
		gtk_list_store_set(store, &iter,
				COL_NAME, name->str,
				COL_TYPE, type->str,
				COL_DESC, desc->str,
				COL_LANG, lang->str,
				-1);
	}

	model = GTK_TREE_MODEL(store);

	
	if(firstrun)
	{
		renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_insert_column_with_attributes(view, -1, "Name", renderer,
				"text", COL_NAME,
				NULL);

		renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_insert_column_with_attributes(view, -1, "Typ", renderer,
				"text", COL_TYPE,
				NULL);

		renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_insert_column_with_attributes(view, -1, "Sprache", renderer,
				"text", COL_LANG,
				NULL);

		renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_insert_column_with_attributes(view, -1, "Beschreibung", renderer,
				"text", COL_DESC,
				NULL);

		firstrun = 0;
	}

	gtk_tree_view_set_model(view, model);

	if(len)
	{
		path = gtk_tree_path_new_from_string("0");
		gtk_tree_view_set_cursor(view, path, NULL, FALSE);
		gtk_tree_path_free(path);
	}

	
	g_object_unref (model);	
}

gint gtk_tree_view_get_single_pos(GtkTreeView* view)
{
	GtkTreePath *path = NULL;
	GtkTreeViewColumn * col = NULL;
	gchar* p = NULL;
	int pos;

	gtk_tree_view_get_cursor(view, &path, &col);

	if(!path)
		return -1;

	p = gtk_tree_path_to_string(path);

	if(!p) return -1;

	pos = atoi(p);

	g_free(p);

	return pos;
}

void gtk_label_clear(GtkLabel* label)
{
	gtk_label_set_text(label, "");
}

void gtk_label_set_aof(GtkLabel* label, GList* list, gint pos)
{
	GString *name, *desc, *longdesc, *type, *year, *author, *field, *language;
	const gchar* text =
		
"<span foreground=\"blue\" size=\"medium\"><b><span color=\"#FF0000\">Name:</span></b> %s (%s)\n\n\
<b><span color=\"#FF0000\">Typ:</span></b> %s\n\n\
<b><span color=\"#FF0000\">Sprache:</span></b> %s\n\n\
<b><span color=\"#FF0000\">Autor:</span></b> %s\n\n\
<b><span color=\"#FF0000\">Jahr:</span></b> %s\n\n\
<b><span color=\"#FF0000\">Beschreibung:</span></b> %s</span>";

	/* id, name, desc, longdesc, type, year, author, path, language
         * 0   1      2     3        4     5      6      7     8
         *     +      +     +                     +      +     +
         */

	name = g_list_nth_data(list, 9*pos + 1);
	desc = g_list_nth_data(list, 9*pos + 2);
	longdesc = g_list_nth_data(list, 9*pos + 3);
	type = g_list_nth_data(list, 9*pos + 4);
	year = g_list_nth_data(list, 9*pos + 5);
	author = g_list_nth_data(list, 9*pos + 6);
	language = g_list_nth_data(list, 9*pos + 8);

	field = g_string_new("");

	g_string_append_printf(field, text, name->str, desc->str, type->str, language->str, author->str, year->str, longdesc->str);


	gtk_label_set_text(label, field->str);
	g_object_set(label, "use-markup", TRUE, NULL);
}

int gtk_exec(const gchar* file, GList* argv)
{
	gint len, i, status;
	gchar **ar, **tmp, *string;

	len = g_list_length(argv);

	ar = g_malloc(sizeof(gchar*));

	if(!ar)
		return -1;

	ar[0] = (gchar*) file;

	for(i = 0; i < len; ++i)
	{
		tmp = g_realloc(ar, sizeof(gchar*) * ( i + 2));

		if(!tmp)
		{
			g_free(ar);
			return -1;
		}

		if(tmp != ar)
			ar = tmp;

		string = g_list_nth_data(argv, i);
		
		ar[i + 1] = string;
	}

	tmp = g_realloc(ar, sizeof(gchar*) * ( i + 2));

	if(!tmp)
	{
		g_free(ar);
		return -1;
	}

	if(tmp != ar)
		ar = tmp;

	ar[i + 1] = NULL;

	switch(fork())
	{
		case -1:
			perror("vfork");
			g_free(ar);
			return -1;
			break;
		case 0:
			execvp(file, ar);
			_exit(EXIT_SUCCESS);
			break;

	}

	wait(&status);
	
	g_free(ar);
	return status; /* 0 ==> success ; !0 ==> error */
}
